#!/bin/ksh
# alarm.sh 1.30 of 08/19/02
# @(#)alarm.sh	1.30
###############################################################################
# alarm.sh 
###############################################################################
# Title:   alarm.sh
#
#          To utilize this script, add the following line to the ONCONFIG file
#          ALARMPROGRAM      <informixdir>/scripts/alarm.sh     or
#          ALARMPROGRAM      <informixdir>/scripts/alarm.<instance>
#          where <informixdir> is replaced by the full value of $INFORMIXDIR
#
# This script mails or pages the Informix DBA's and/or operations if an alarm
# condition arises.  Another primary function is to back up the transaction
# (logical) logs once a "log full" alarm is received.
#
# This shell is designed for single and multiple instance (engine) servers.
# For single instance servers, use alarm.sh in the configuration file.  If
# there is more than one instance on a server, it is recommended that the name
# of this script be different for each instance in each onconfig.  The shell
# as written expects that name to be alarm.<instance>, so that the onconfig
# file name (onconfig.<instance>) and INFORMIXSERVER value (<instance>_shm)
# can be determined by the script.
#
# If the /usr/informix/alarm.nomail file exists, no mail or notification 
# is sent from this program.  This file should exist on testing machines.
###############################################################################

# Set trace on if necessary for debugging.
# set -o xtrace

# Obtain parameters passed by Informix.
EVENT_SEVERITY=$1
EVENT_CLASS=$2
EVENT_MSG="$3"
EVENT_ADD_TEXT="$4"
EVENT_FILE="$5"

# Set contact and shell variables.
DBA_MAIL="dba@s1.com"
OPS_PIN=7992239
OPS_PAGER=skytel.com
MAILFILE=${INFORMIXDIR:-/usr/informix}/alarm.nomail
if [ -r ${MAILFILE} ]
then
	MAILFLAG="off"
else
	MAILFLAG="on"
fi
ENV=`basename $0 | cut -d. -f2`
PROG=`basename $0`
HOST=`hostname`

# Set up variables for temporary files and alarm log used to store all alarms.
TMPFILE=${TMPDIR:-/tmp}/alarm${ENV}.$$
PAGEFILE=${TMPDIR:-/tmp}/page${ENV}.$$
if [ "${ENV}" = "sh" ]
then
	ALARMLOG=${INFORMIXDIR:-/usr/informix}/alarm.log
else
	ALARMLOG=/db_activity/${ENV}/alarm.log
fi

# This one program is shared by all of the event alarms.
# Create a temporary file with data passed from the engine.
echo "$PROG got event $EVENT_CLASS" > ${TMPFILE}
echo "	severity       : $EVENT_SEVERITY"   >> ${TMPFILE}
echo "	message        : $EVENT_MSG"        >> ${TMPFILE}
echo "	additional text: $EVENT_ADD_TEXT"   >> ${TMPFILE}
echo "	reference file : $EVENT_FILE"       >> ${TMPFILE}
echo                                        >> ${TMPFILE}

# Place alarm information into alarm log unless class type is one that repeats a lot.
case "$EVENT_CLASS" in
	6|15|21)  # Do not update alarm log at this point - done later if necessary.
		  ;;
	*)	  # Update alarm log.
		  echo `date` >> ${ALARMLOG}
		  cat ${TMPFILE} >> ${ALARMLOG}
		  ;;
esac

# Try to prevent multiple alarms (usually class 6, 15, and 21) from overwhelming the processor.
# When the alarm shell is called, 2 processes are spawned and the grep is a third process.
# Take action if there are more than 2 alarms already being processed.
NUMALARMS=`ps -ef | grep -c "alarm.${ENV}"`
if [ ${NUMALARMS} -gt 5 ]
then
	if [ $EVENT_CLASS -eq 18 ]
	then
		# Class 18 (log backup completed) can appear while log full alarm is still active.
		# The alarm has already been logged in above case statement.
		rm -f ${TMPFILE}
		exit 0
	else
		# The alarm is one of many duplicates, so just add to log file to have a record of it.
		echo `date` >> ${ALARMLOG}
		echo "Multiple alarms - class $EVENT_CLASS, severity $EVENT_SEVERITY." >> ${ALARMLOG}
		rm -f ${TMPFILE}
		exit 1
	fi
fi

# Send email and/or notify operations based on the severity of the alarm detected.
case "$EVENT_SEVERITY" in
	1|2)	# Do not mail or page - informational only.
		;;
	3)	# Attention alarm - mail data to production DBA's unless class 6, 15, 21, or 23.
		# Also, send a major (not critical) alarm to operations.
		case "$EVENT_CLASS" in
			6|15|21)  # Do not send mail or notify operations at this point - done later if necessary.
				  # Class  6 is sent when there is a non-fatal subsystem failure.
				  # Class 15 is sent when there is a data replication failure.
				  # Class 21 is sent when there is a resource overflow.
		  		  ;;
			23)	  # Do not send mail - action will be taken automatically.
				  # Class 23 is sent when a logical log has filled up.
		  		  ;;
			*)	  # Send mail to DBA team and notify operations for all other classes with this severity.
				  if [ "${MAILFLAG}" = "on" ]
				  then
					echo "Attention alarm for Informix database on host ${HOST} on `date`." > ${PAGEFILE}
					head -3 ${TMPFILE} >> ${PAGEFILE}
					echo "Contact production DBA immediately if there are application issues." >> ${PAGEFILE}
					# Send an HP IT/O message if IT/O is installed, otherwise page operations.
					if [ -s /opt/OV/bin/OpC/opcmsg ]
					then
						/opt/OV/bin/OpC/opcmsg severity=major application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
					else
						mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
					fi
					mailx -s "$PROG called for host $HOST, event $EVENT_CLASS, severity $EVENT_SEVERITY, on `date`" ${DBA_MAIL} < ${TMPFILE}
					rm -f ${PAGEFILE}
				  fi
		  		  ;;
		esac
		;;
	4|5)	# Emergency or fatal alarm - mail data to production DBA's
		# and notify operations with a critical alarm.
		case "$EVENT_CLASS" in
			6|15|21)  # Do not send notification at this point - done later if necessary.
		  		  ;;
			*)	  # Send notification for all other classes with this severity.
				  if [ "${MAILFLAG}" = "on" ]
				  then
					  echo "Emergency or fatal alarm for Informix database on host ${HOST} on `date`." > ${PAGEFILE}
					  head -3 ${TMPFILE} >> ${PAGEFILE}
					  echo "Contact production DBA immediately." >> ${PAGEFILE}
					  # Send an HP IT/O message if IT/O is installed, otherwise page operations.
					  if [ -s /opt/OV/bin/OpC/opcmsg ]
					  then
						  /opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
					  else
						  mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
					  fi
					  mailx -s "$PROG called for host $HOST, event $EVENT_CLASS, severity $EVENT_SEVERITY, on `date`" ${DBA_MAIL} < ${TMPFILE}
					  rm -f ${PAGEFILE}
				  fi
		  		  ;;
		esac
		;;
esac

# Special processing when alarm class is 6, 15 or 21 to prevent duplicate alarms from swamping mail.
# If alarm class is 23 (log filled), perform a logical (transaction) log backup to UNIX file system.
# If any other class, simply exit.  Our work is complete.
case "$EVENT_CLASS" in
       6|15|21) # Special processing for these classes since they can create many alarms quickly.
		# Ignore if severity is only 1 or 2.
		if [ $EVENT_SEVERITY -lt 3 ]
		then
			rm -f ${TMPFILE}
			exit 0
		fi
		FILENO=$EVENT_CLASS
		# Determine if this is a new or "existing" issue by checking to see if a
		# previous event file with this class is in /tmp.  If not, it is a new issue.
		if [ -s /tmp/event${ENV}${FILENO}.* ]
		then
			# If it has been more than 60 minutes since the same alarm, I consider it
			# a new issue.  This is a design decision. 
			# Get month, day, and hour when event file was created.
			ls -lt /tmp/event${ENV}${FILENO}.* > /tmp/alarm${ENV}${FILENO}.ls
			FILEMO=`head -1 /tmp/alarm${ENV}${FILENO}.ls | awk '{print $6}'`
			FILEDY=`head -1 /tmp/alarm${ENV}${FILENO}.ls | awk '{print $7}'`
			FILEHR=`head -1 /tmp/alarm${ENV}${FILENO}.ls | awk '{print $8}' | awk -F: '{print $1}'`
			# Get month, day, and hour of current date.
			DATEMO=`date +%b`
			DATEDY=`date +%d`
			DATEHR=`date +%H`
			# First check the obvious.  Are months or days not the same or is the file is very old?
			# The FILEHR test for 1900 finds older files that have no time stamp.
			if [ "${FILEMO}" != "${DATEMO}" -o $FILEHR -gt 1900 -o $FILEDY -ne $DATEDY ]
			then
				ISSUE="new"
			else
				# Get minute of file and current date.
				FILEMI=`head -1 /tmp/alarm${ENV}${FILENO}.ls | awk '{print $8}' | awk -F: '{print $2}'`
				DATEMI=`date +%M`
				# Add one to hour of file for testing later.
				FILEHR1=`expr $FILEHR + 1`
				if [ $FILEHR1 -gt 23 ]
				then
					FILEHR1=`expr $FILEHR - 23`
				fi
				# The hours and minutes are tested, and if not within 60 minutes, this is a new issue.
				if [ $FILEHR -ne $DATEHR ]
				then
					if [ $DATEMI -gt $FILEMI -o $FILEHR1 -lt $DATEHR ]
					then
						ISSUE="new"
					else
						ISSUE="old"
					fi
				else
					ISSUE="old"
				fi
			fi
			rm -f /tmp/alarm${ENV}${FILENO}.ls
		else
			# No existing event file. This is the first time this alarm went off.
			ISSUE="new"
		fi
		if [ "${ISSUE}" = "new" ]
		then
			# This is a new issue - update the alarm log.
			echo `date` >> ${ALARMLOG}
			cat ${TMPFILE} >> ${ALARMLOG}
			# Create file containing status information for review later.
			rm -f /tmp/event${ENV}${FILENO}.*
			STFILE=/tmp/event${ENV}${FILENO}.`date +%H`
			cp ${TMPFILE} ${STFILE}
			onstat -a >> ${STFILE}
			date >> ${STFILE}
			# Notify operations and send email to DBA's if notification wanted.
			if [ "${MAILFLAG}" = "on" ]
			then
				if [ $EVENT_SEVERITY -gt 3 ]
				then
					echo "Emergency or fatal alarm for Informix database on host ${HOST} on `date`." > ${PAGEFILE}
					head -3 ${TMPFILE} >> ${PAGEFILE}
					echo "Contact production DBA immediately." >> ${PAGEFILE}
					# Send an HP IT/O message if IT/O is installed, otherwise page operations.
					if [ -s /opt/OV/bin/OpC/opcmsg ]
					then
						/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
					else
						mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
					fi
					rm -f ${PAGEFILE}
				else
					echo "Attention alarm for Informix database on host ${HOST} on `date`." > ${PAGEFILE}
					head -3 ${TMPFILE} >> ${PAGEFILE}
					echo "Contact production DBA immediately if there are application issues." >> ${PAGEFILE}
					# Send an HP IT/O message if IT/O is installed, otherwise page operations.
					if [ -s /opt/OV/bin/OpC/opcmsg ]
					then
						/opt/OV/bin/OpC/opcmsg severity=major application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
					else
						mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
					fi
					rm -f ${PAGEFILE}
				fi
				mailx -s "$PROG called for host $HOST, event $EVENT_CLASS, severity $EVENT_SEVERITY, on `date`" ${DBA_MAIL} < ${TMPFILE}
			fi
		else
			# Since the event file is less than 1 hour old, this is an
			# existing issue.  Do not notify operations or email DBA's.
			# Just log the alarm to the alarm log.
			echo `date` >> ${ALARMLOG}
			echo "Existing class $EVENT_CLASS issue - no notification needed." >> ${ALARMLOG}
		fi
		rm -f ${TMPFILE}
		exit 0
		;;
	23)	# Logical log(s) need to be backed up - continue with shell.
		continue
		;;
	*)	# No other action required by this shell - exit.
		rm -f ${TMPFILE}
		exit 0
		;;
esac

# Logical Log Backup Section!

# The alarm.out file contains the results of the logical log backups.
if [ "${ENV}" = "sh" ]
then
	ALARMOUT=${INFORMIXDIR:-/usr/informix}/logs/alarm.out
	MSGENV=""
else
	ALARMOUT=/db_activity/${ENV}/logs/alarm.out
	MSGENV="${ENV}"
fi
echo "==========================================================" >> ${ALARMOUT}
echo `date` >> ${ALARMOUT}

# Set more shell variables required to run the logical log backups.
BACKUPCURR=n			# Back up current log?  Default is no.
CLOSELOG=n			# Close transaction log?  Should be no.
				# If yes, another alarm program will start.
START=`date '+%m/%d/%y %H:%M'`

# Capture Informix information from .profile of informix ID and
# execute it so that logs can be backed up.
INFHOME=`grep informix /etc/passwd | head -1 | cut -d: -f6`
grep INFORMIXDIR ${INFHOME}/.profile > ${INFHOME}/.tmpalm.$$
if [ "${ENV}" = "sh" ]
then
        grep INFORMIXSERVER ${INFHOME}/.profile >> ${INFHOME}/.tmpalm.$$
        grep ONCONFIG ${INFHOME}/.profile >> ${INFHOME}/.tmpalm.$$
else
	export INFORMIXSERVER=${ENV}_shm
        export ONCONFIG=onconfig.${ENV}
fi
. ${INFHOME}/.tmpalm.$$
rm -f ${INFHOME}/.tmpalm.$$
export PATH=$PATH:${INFORMIXDIR}/bin:/usr/contrib/bin

BACKUPDEV=`grep LTAPEDEV ${INFORMIXDIR}/etc/${ONCONFIG} | grep -v '#LTAPEDEV' | awk '{print $2}'`
LOGSDIR=`dirname $BACKUPDEV`

# Make sure no other ontape is running or "mutex" file does not exist.  If 
# either, notify operations and exit.  If not, create "mutex" file in /tmp.
if [ -f /tmp/ontape.L${ENV} ]
then
	echo "Logical log backup failure because another log backup is running." >> ${ALARMOUT}
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "${MSGENV} Logical log backup failed on host ${HOST} on `date`." > ${PAGEFILE}
		echo "Another log backup is running." >> ${PAGEFILE}
		echo "Call DBA only if this happens for more than 30 minutes." >> ${PAGEFILE}
		# Send an HP IT/O message if IT/O is installed, otherwise page operations.
		if [ -s /opt/OV/bin/OpC/opcmsg ]
		then
			/opt/OV/bin/OpC/opcmsg severity=major application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
		else
			mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
		fi
		mailx -s "Log backup failure on host ${HOST} on `date` because another ${MSGENV} log backup is running" ${DBA_MAIL} < ${PAGEFILE}
		rm -f ${PAGEFILE}
	fi
	rm -f ${TMPFILE}
	exit 2
else
	# This prevents other log backups from running (from alarm and any other scripts).
	echo " " > /tmp/ontape.L${ENV}
	echo ${BACKUPCURR} >> /tmp/ontape.L${ENV}
fi

# No need to back up logs if device is /dev/null, but run ontape -a anyway in
# case reserved page value is different than the onconfig LTAPEDEV value.
echo ${BACKUPDEV} | grep "dev/null" > /dev/null
if [ $? -eq 0 ]
then
	echo "Logs are going to /dev/null - no need to continue with backup." >> ${ALARMOUT}
	# Run ontape command anyway to make sure log is freed since onconfig value may be
	# different than the value of LTAPEDEV value in reserved page.
	ontape -a < /tmp/ontape.L${ENV} >/tmp/alarm.ontape${ENV} 2>&1
	rm -f ${TMPFILE}
	rm -f /tmp/ontape.L${ENV}
	exit 3
fi

# Perform onstat command to make sure that the database is online.
# Notify operations if the database is down.
INF_STAT=`${INFORMIXDIR}/bin/onstat - | grep -c On-Line` 
if [ ${INF_STAT} -eq 0 ]
then
	echo "Log backup failure because the database is down or quiescent." >> ${ALARMOUT}
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "Informix database ${MSGENV} is down on host ${HOST} on `date`." > ${PAGEFILE}
		echo "Notification from log backup portion of alarm shell." >> ${PAGEFILE}
		echo "Contact production DBA immediately." >> ${PAGEFILE}
		# Send an HP IT/O message if IT/O is installed, otherwise page operations.
		if [ -s /opt/OV/bin/OpC/opcmsg ]
		then
			/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
		else
			mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
		fi
		mailx -s "Log backup failure on host ${HOST} on `date` because the ${MSGENV} database is down or quiescent" ${DBA_MAIL} < ${PAGEFILE}
		rm ${PAGEFILE}
	fi
	rm -f ${TMPFILE}
	rm -f /tmp/ontape.L${ENV}
	exit 4
fi

# One option is to close the current log before archive begins - default is no.
case ${CLOSELOG} in
	[yY]*)	echo "Closing current transaction log." >> ${ALARMOUT}
		onmode -l 	
		;;
	*)
		;;
esac

# Set shell variables for log storage file name pieces. The separate date
# variables are needed because SCCS Identification Strings use % characters.
HH=`date '+%H'`
MI=`date '+%M'`
SS=`date '+%S'`
PREVSUFF=${HH}${MI}${SS}.prev

# Get rid of log files that are older than 72 hours to free up space in the
# log storage file system.
if [ -d ${LOGSDIR} ]
then
	find ${LOGSDIR} -name "logfile.[0-9]*" -mtime +2 -exec rm -f {} \;
fi

# Look for log backup that has not been moved. If one is found, rename and compress it!
if [ -r ${BACKUPDEV} ]
then
	ls -l ${BACKUPDEV} >> ${ALARMOUT}
	echo "Found previous transaction log backup. Moving it to ${BACKUPDEV}.${PREVSUFF} " >> ${ALARMOUT}
	echo "and then will try to gzip the file." >> ${ALARMOUT}
	mv ${BACKUPDEV} ${BACKUPDEV}.${PREVSUFF}
	gzip -f ${BACKUPDEV}.${PREVSUFF} >/tmp/gzip.out 2>&1
fi

# Perform the actual log backup after initializing output device.
cat /dev/null > ${BACKUPDEV}
chmod 664 ${BACKUPDEV}
if [ ! -w ${BACKUPDEV} ]
then 
	echo "Log backup failure because backup device ${BACKUPDEV} cannot be made ready." >> ${ALARMOUT}
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "${MSGENV} Logical log backup failed on host ${HOST} on `date`." > ${PAGEFILE}
		echo "Backup device ${BACKUPDEV} cannot be made ready." >> ${PAGEFILE}
		echo "Contact production DBA immediately." >> ${PAGEFILE}
		# Send an HP IT/O message if IT/O is installed, otherwise page operations.
		if [ -s /opt/OV/bin/OpC/opcmsg ]
		then
			/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
		else
			mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
		fi
		mailx -s "Log backup failure on host ${HOST} on `date` because ${MSGENV} backup device cannot be made ready" ${DBA_MAIL} < ${PAGEFILE}
		rm -f ${PAGEFILE}
	fi
	rm -f ${TMPFILE}
	rm -f /tmp/ontape.L${ENV}
	exit 5
fi

echo "Backing up Informix logical logs to ${BACKUPDEV}." >> ${ALARMOUT}

# Old way of getting log number(s) being backed up - reinstated to prevent the
# reliance on shared memory connection being up (was down a couple of times).
# FLOGNUM=`onstat -l | grep U---- | sort -kn4 | awk '{print $4}' | head -1`
# TLOGNUM=`onstat -l | grep U---- | sort -kn4 | awk '{print $4}' | tail -1`

onstat -l | grep U--- | sort -kn4 | awk '{print $4}' > /tmp/alarm.sed.$$

# Determine number of first log that will be backed up and create file name.
# dbaccess sysmaster <<EOF
# unload to /tmp/alarm.out.$$ delimiter " "
# select uniqid from syslogs where is_backed_up = 0
# order by uniqid
# EOF
# sed -e 's/ //g' /tmp/alarm.out.$$ > /tmp/alarm.sed.$$

FLOGNUM=`head -1 /tmp/alarm.sed.$$`
TLOGNUM=`tail -1 /tmp/alarm.sed.$$`

LLOGNUM=`expr ${TLOGNUM} - 1`
#LLOGNUM=${TLOGNUM}
LOGSUFF=${FLOGNUM}-${LLOGNUM}
if [ "${FLOGNUM}" -eq "" -o "${TLOGNUM}" -eq "" ]
then
	echo "Unusual values in onstat statement for log numbers:" >> ${ALARMOUT}
	cat /tmp/alarm.sed.$$ >> ${ALARMOUT}
	echo "End of values created by onstat statement for log numbers." >> ${ALARMOUT}
fi
#rm -f /tmp/alarm.out.$$
rm -f /tmp/alarm.sed.$$

# Perform the logical log backup!
ontape -a < /tmp/ontape.L${ENV} >/tmp/alarm.ontape${ENV} 2>&1

STAT=$?
if [ "$STAT" -ne 0 ]
then
	echo "Logical log backup failure with status ${STAT}." >> ${ALARMOUT}
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "${MSGENV} Logical log backup failed on host ${HOST} on `date`." > ${PAGEFILE}
		echo "The ontape command has failed with status ${STAT}." >> ${PAGEFILE}
		echo "Contact production DBA immediately." >> ${PAGEFILE}
		# Send an HP IT/O message if IT/O is installed, otherwise page operations.
		if [ -s /opt/OV/bin/OpC/opcmsg ]
		then
			/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${PAGEFILE}`" msg_grp=Database
		else
			mail ${OPS_PIN}@${OPS_PAGER} < ${PAGEFILE}
		fi
		mailx -s "Log backup failure on host ${HOST} on `date` because ${MSGENV} ontape status was ${STAT}" ${DBA_MAIL} < ${PAGEFILE}
		rm -f ${PAGEFILE}
	fi
	rm -f ${TMPFILE}
	rm -f /tmp/ontape.L${ENV}
	exit 6
fi

# Insure that there is no log backup file (or compressed version) with the same name
# which the existing backup log file name will be renamed to.
if [ -f ${BACKUPDEV}.${LOGSUFF} ]
then
	ls -ltrab ${BACKUPDEV}.${LOGSUFF} >> ${ALARMOUT}
	echo "Renaming existing ${BACKUPDEV}.${LOGSUFF} to ${BACKUPDEV}.${LOGSUFF}.dupl." >> ${ALARMOUT}
	mv ${BACKUPDEV}.${LOGSUFF} ${BACKUPDEV}.${LOGSUFF}.dupl
fi
if [ -f ${BACKUPDEV}.${LOGSUFF}.gz ]
then
	ls -ltrab ${BACKUPDEV}.${LOGSUFF}.gz >> ${ALARMOUT}
	echo "Renaming existing ${BACKUPDEV}.${LOGSUFF}.gz to ${BACKUPDEV}.${LOGSUFF}.dupl.gz." >> ${ALARMOUT}
	mv ${BACKUPDEV}.${LOGSUFF}.gz ${BACKUPDEV}.${LOGSUFF}.dupl.gz
fi

# Rename backup file to include first log backed up and time stamp, then compress it.
# If compress fails, only send email as this is not a critical issue.
echo "Renaming and compressing ${BACKUPDEV} to ${BACKUPDEV}.${LOGSUFF}." >> ${ALARMOUT}
mv ${BACKUPDEV} ${BACKUPDEV}.${LOGSUFF}
gzip -f ${BACKUPDEV}.${LOGSUFF} >/tmp/gzip.out 2>&1
STAT=$?
if [ "$STAT" -ne 0 ]
then
	echo "${MSGENV} Logical log backup compress (gzip) failure with status ${STAT} for log(s) ${LOGSUFF}." >> ${ALARMOUT}
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "${MSGENV} Logical log compress (gzip) failure on host ${HOST} on `date`." > ${TMPFILE}
		echo "The gzip command has failed for log(s) ${LOGSUFF} with status ${STAT}." >> ${TMPFILE}
		mailx -s "${MSGENV} Log gzip command failure on host $HOST on `date`" ${DBA_MAIL} < ${TMPFILE}
	fi
fi

# Remove input file to log backup. It also serves as the "mutex" file that prevents
# multiple log backups running at the same time (from alarm and any other scripts).
rm -f /tmp/ontape.L${ENV}

# Note in the alarm output that the transaction log backup has been completed.
END=`date '+%m/%d/%y %H:%M'`
echo "${MSGENV} Informix transaction log backup completed on ${HOST}." > ${TMPFILE}
echo "Start: ${START}  End: ${END}  First log is: ${FLOGNUM}." >> ${TMPFILE}
echo "File name should be ${BACKUPDEV}.${LOGSUFF}.gz in ${LOGSDIR}." >> ${TMPFILE}

cat ${TMPFILE} >> ${ALARMOUT}
rm -f ${TMPFILE}

exit 0
