#!/bin/ksh
# checkdb.sh 1.18 of 12/09/03
# @(#)checkdb.sh	1.18
##############################################################################
# checkdb.sh
##############################################################################
# Script that checks database to make sure it is up.  It is run every
# 15 minutes from Maestro.  It sends a message to OpenView/IPO if the
# database is down, so Data Center Operations can take action. 
#
# For recovery purposes, it forces a back up of the logical logs by doing
# an onmode -l command, but only if no other log backup is currently running,
# and only if the movelogs flag parameter is "yes".
# 
# If the last parameter is "yes", this shell also checks the online.log for the
# last checkpoint.  If the log has not been updated in the last 30 minutes, a
# critical alarm is sent to Data Center Operations.
#
# Two or three parameters are passed to the shell.  The first is the instance
# name if this is a multi-instance server.  The second (or first) contains
# the value of the movelogs flag ("yes" or "no").  The third (or second)
# contains the value of the check_log flag ("yes" or "no").
#
# If the /usr/informix/alarm.nomail file exists, no mail or notification 
# is sent from this program.  This file should exist on testing machines.
##############################################################################

# Set up environment.

# Capture Informix information from .profile of informix ID.
INFHOME=`grep informix /etc/passwd | head -1 | cut -d: -f6`
grep INFORMIXDIR ${INFHOME}/.profile > ${INFHOME}/.tmpchk.$$
if [ $# -le 2 ]
then
        grep INFORMIXSERVER ${INFHOME}/.profile >> ${INFHOME}/.tmpchk.$$
        grep ONCONFIG ${INFHOME}/.profile >> ${INFHOME}/.tmpchk.$$
	export MULTI="no"
else
        export INFORMIXSERVER=${1}_shm
        export ONCONFIG=onconfig.${1}
	export INFNAME=${1}
	export MULTI="yes"
	shift 1
fi
. ${INFHOME}/.tmpchk.$$
rm ${INFHOME}/.tmpchk.$$
export PATH=$PATH:$INFORMIXDIR/bin

# Movelogs flag is passed to the shell. The default is "no". The movelogs 
# flag is "yes" if this shell is to move the logs after checking the status
# of the database (for disaster recovery purposes).
if [ "${1}" = "no" ]
then
        export MOVELOGS="no"
else
        if [ "${1}" = "yes" ]
        then
                export MOVELOGS="yes"
        else
                export MOVELOGS="no"
        fi
fi

if [ "${2}" = "no" ]
then
        export CHECKLOG="no"
else
        if [ "${2}" = "yes" ]
        then
                export CHECKLOG="yes"
        else
                export CHECKLOG="yes"
        fi
fi

DBA_MAIL="dba@s1.com"
HOST=`hostname`
if [ "${MULTI}" = "yes" ]
then
        LOG_DIR="/db_activity/${INFNAME}/logs"
else
        LOG_DIR="${INFORMIXDIR}/logs"
fi

TMPFILE=${TMPDIR:-/tmp}/checkdb.$$
MAILFILE=${INFORMIXDIR:-/usr/informix}/alarm.nomail
if [ -r ${MAILFILE} ]
then
	MAILFLAG="off"
else
	MAILFLAG="on"
fi

cd ${LOG_DIR}

exec >> ${LOG_DIR}/checkdb.out 2>&1

# Check to see if the database is up using the onstat - command.
# Send messages and exit if database is down.

echo "=========================================================="

INF_STAT=`$INFORMIXDIR/bin/onstat - | grep -c On-Line`  # DOWN if = 0
if [ ${INF_STAT} -eq 0 ]
then 
	echo "Informix database ${INFORMIXSERVER} is down on host ${HOST} on `date`."
	env
	if [ "${MAILFLAG}" = "on" ]
	then
		echo "Informix database ${INFORMIXSERVER} is down on host ${HOST} on `date`." > ${TMPFILE}
		echo "Contact production DBA immediately." >> ${TMPFILE}
		if [ -s /opt/OV/bin/OpC/opcmsg ]
		then
			/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${TMPFILE}`" msg_grp=Database
		fi
		mailx -m -s "Database ${INFORMIXSERVER} is down on host ${HOST} on `date`" ${DBA_MAIL} < ${TMPFILE}
		sleep 3
	fi
	rm -f ${TMPFILE}
	exit 1
else
	INF_STAT=`$INFORMIXDIR/bin/onstat - | grep -c WARNING`  # DOWN if > 0
	if [ ${INF_STAT} -eq 0 ]
	then 
		echo "Informix database ${INFORMIXSERVER} is up on host ${HOST} on `date`."
		onstat -
	else
		echo "Informix database ${INFORMIXSERVER} is down on host ${HOST} on `date`."
		env
		if [ "${MAILFLAG}" = "on" ]
		then
			echo "Informix database ${INFORMIXSERVER} is down on host ${HOST} on `date`." > ${TMPFILE}
			echo "Contact production DBA immediately." >> ${TMPFILE}
			if [ -s /opt/OV/bin/OpC/opcmsg ]
			then
				/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${TMPFILE}`" msg_grp=Database
			fi
			mailx -m -s "Database ${INFORMIXSERVER} is down on host ${HOST} on `date`" ${DBA_MAIL} < ${TMPFILE}
			sleep 3
		fi
		rm -f ${TMPFILE}
		exit 1
	fi
fi

# Do this next section only if requested by command line (CHECKLOG value).
# If there are no recent checkpoints at all in the online log, there may be a problem
# with the database. Send messages and exit if database has no recent checkpoints.
# Using tail command instead of onstat -m to obtain last 200 lines of online log.

if [ "${CHECKLOG}" = "yes" ]
then
	tail -200 ../online.log | grep "Checkpoint Completed" > ${TMPFILE}
	if [ ! -s ${TMPFILE} ]
	then
		echo "Informix database on host ${HOST} on `date` has no recent checkpoint."
		tail -25 ../online.log
		env
		if [ "${MAILFLAG}" = "on" ]
		then
			echo "Informix database ${INFORMIXSERVER} on host ${HOST} on `date` has no recent checkpoint." > ${TMPFILE}
			echo "Contact production DBA immediately if there are application issues." >> ${TMPFILE}
			if [ -s /opt/OV/bin/OpC/opcmsg ]
			then
				/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${TMPFILE}`" msg_grp=Database
			fi
			mailx -m -s "Database ${INFORMIXSERVER} has no recent checkpoint on host ${HOST} on `date`" ${DBA_MAIL} < ${TMPFILE}
			sleep 3
		fi
		rm -f ${TMPFILE}
		exit 2
	fi

# Now, determine if latest checkpoint is within 15 minutes of current date and time.
# First, obtain hour and minute of last checkpoint and add 15 minutes to it.

	FILEHR=`cat ${TMPFILE} | tail -1 | cut -d: -f1`
	FILEMN=`cat ${TMPFILE} | tail -1 | cut -d: -f2`
	if [ ${FILEMN} -lt 45 ]
	then
		FILEMN=`expr $FILEMN + 15`
	else
		FILEMN=`expr $FILEMN - 45`
		FILEHR=`expr $FILEHR + 1`
		if [ ${FILEHR} -gt 23 ]
		then
			FILEHR=`expr $FILEHR - 24`
		fi	
	fi	

# Obtain current date and add 1 to hour for tests below.

	DATEHR=`date +%H`
	DATEMN=`date +%M`
	DATEHR1=`expr $DATEHR + 1`
	if [ ${DATEHR1} -gt 23 ]
	then
		DATEHR1=`expr $DATEHR1 - 24`
	fi	

# The hour and minutes of the last checkpoint is compared to the current date and time.
# If they are not within 15 minutes, there is probably a problem with the database or
# application.  If so, send messages to Data Center and DBA team and exit.

	export LOGCHECK="notsure"
	if [ ${FILEHR} -eq ${DATEHR} ]
	then
		if [ ${FILEMN} -lt ${DATEMN} ]
		then
			echo "Informix database on host ${HOST} on `date` has no recent checkpoint."
			tail -25 ../online.log
			env
			if [ "${MAILFLAG}" = "on" ]
			then
				echo "Informix database ${INFORMIXSERVER} on host ${HOST} on `date` has no recent checkpoint." > ${TMPFILE}
				echo "Contact production DBA immediately if there are application issues." >> ${TMPFILE}
				if [ -s /opt/OV/bin/OpC/opcmsg ]
				then
					/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${TMPFILE}`" msg_grp=Database
				fi
				mailx -m -s "Database ${INFORMIXSERVER} has no recent checkpoint on host ${HOST} on `date`" ${DBA_MAIL} < ${TMPFILE}
				sleep 3
			fi
			rm -f ${TMPFILE}
			exit 2
		fi
		export LOGCHECK="success"
	fi

	if [ ${FILEHR} -ne ${DATEHR1} ] && [ "${LOGCHECK}" != "success" ]
	then
		echo "Informix database on host ${HOST} on `date` has no recent checkpoint."
		tail -25 ../online.log
		env
		if [ "${MAILFLAG}" = "on" ]
		then
			echo "Informix database ${INFORMIXSERVER} on host ${HOST} on `date` has no recent checkpoint." > ${TMPFILE}
			echo "Contact production DBA immediately if there are application issues." >> ${TMPFILE}
			if [ -s /opt/OV/bin/OpC/opcmsg ]
			then
				/opt/OV/bin/OpC/opcmsg severity=critical application=DBA object=Informix msg_text="`cat ${TMPFILE}`" msg_grp=Database
			fi
			mailx -m -s "Database ${INFORMIXSERVER} has no recent checkpoint on host ${HOST} on `date`" ${DBA_MAIL} < ${TMPFILE}
			sleep 3
		fi
		rm -f ${TMPFILE}
		exit 2
	fi

# End of CHECKLOG section
fi 

# Finally, generate an alarm to cause the logical log(s) to be backed up
# for use in disaster recovery. This is only done if movelogs flag is "yes",
# and is done by moving the logs using onmode.
if [ "${MOVELOGS}" = "no" ]
then
	echo "Informix database ${INFORMIXSERVER} logs not moved on host ${HOST} on `date`."
	rm -f ${TMPFILE}
	exit 0
fi

if [ "${MULTI}" = "yes" ]
then
	ENV=${INFNAME}
else
	ENV=sh
fi

# Do this only if no other ontape (logical log backup) is running for this instance.
# Run from another shell to prevent this shell from hanging in case of hung checkpoint.

/usr/informix/scripts/checkdb_log.sh ${ENV} ${HOST} &

#if [ ! -f /tmp/ontape.L${ENV} ]
#then
#        onmode -l 
#        sleep 2
#        onmode -c
#	echo "Informix database ${INFORMIXSERVER} logs moved on host ${HOST} on `date`."
#else
#	echo "Informix database ${INFORMIXSERVER} log backup already in progress on host ${HOST} on `date`."
#fi

rm -f ${TMPFILE}
exit 0

