#!/bin/ksh
# lookatspace.sh 1.12 of 10/28/03
# @(#)lookatspace.sh	1.12
################################################################################
# lookatspace.sh 
################################################################################
# This script determines if a table is about to need another extent that exceeds
# the amount of space remaining in the dbspace containing that table.  It can
# handle tables that are fragmented across dbspaces.
#
# It also monitors the blob space, if used.
# 
# Email is sent to the DBA team for action if any possible issues are found.
#
# Either none, one, or two parameters are passed to the shell.  If the first is
# "MD", this means that there is more than one production database in the
# instance and hence multiple runs of the look at space main routines will be
# done for these multiple databases.  This parameter is then shifted out.
#
# One or no more parameters are then passed to the shell. 
#
# If a parameter is passed, this is a multi-instance server and the value
# of the parameter is the name of the instance being used here.
#
# Typically run from maestro once per week for production.
################################################################################

# Function to actually perform the look at space functions.  Added to make it
# easier to maintain code since adding multiple database in instance
# functionality.

function do_lookatspace
{
# Write header lines to output "report".
echo " "
echo "========================================================================="
echo " "
echo "Start of lookatspace run on `date` for ${INFORMIXSERVER}, database ${DBNAME}"

# Create first lines of two portions of output "report".
echo "DBSpace Status:" > ${DBSFILE}
echo "---------------" >> ${DBSFILE}
echo "DBSpace  Free Pages   % Used" >> ${DBSFILE}
echo "Table Space Status:" > ${TBLFILE}
echo "-------------------" >> ${TBLFILE}

cat /dev/null > ${TMPFILE}
cat /dev/null > ${MAILFILE}

# Obtain amount of space remaining in each dbspace using sysmaster and
# reformat file for use in read statement below.  Discard dbspaces that
# will never have tables or are special (log, temp, tmp, and blob).
dbaccess sysmaster << EOF >>checkspc.out  2>>checkspc.err
unload to "${TMPSPC}" delimiter " "
select name, trunc (sum (nfree), 0),
round (((sum (nfree)) / sum (chksize)) * 100, 2)
from sysdbspaces d, syschunks c
where d.dbsnum = c.dbsnum
group by 1
order by 1;
EOF

sed 's/\.0//' ${TMPSPC} > ${TMPSPC}.1
sed 's/\..*//' ${TMPSPC}.1 > ${TMPSPC}.2
egrep -v "log|tmp|blob|temp" ${TMPSPC}.2 > ${TMPSPC}

# Separate out the blob dbspaces into a different file to be read later.
egrep "blob" ${TMPSPC}.2 > ${TMPSPC}.blob

# Obtain which non-fragmented tables are in which dbspace using systables.
dbaccess ${DBNAME} << EOF >>checkspc.out  2>>checkspc.err
unload to "${TMPTBL}"
select dbinfo ('DBSPACE', partnum), tabname
from systables where tabid >99 and partnum > 0
order by 1;
EOF

# Obtain which fragmented tables are in which dbspace using sysfragments.
dbaccess ${DBNAME} << EOF >>checkspc.out  2>>checkspc.err
unload to "${TMPFRAG}"
select dbspace, tabname, f.partn
from sysfragments f, systables t
where fragtype = "T" and f.tabid = t.tabid
order by 1;
EOF

# Go through dbspace's and the tables in that dbspace.  If any table is close
# to requesting another extent that will exhaust the free space in the
# dbspace, warn the DBA team via email.  If none are close, simply report
# any other instances of another extent needing all remaining space.

while read dbspace amtfree percentfree
do
	DBSPACE="^${dbspace}|"
	grep ${DBSPACE} ${TMPTBL} > ${TMPTBL}.list
	grep ${DBSPACE} ${TMPFRAG} > ${TMPFRAG}.list
	# Reformat files for use in read statments below.
	sed 's/|/ /g' ${TMPTBL}.list > ${TMPTBL}.read
	sed 's/|/ /g' ${TMPFRAG}.list > ${TMPFRAG}.read
	percentused=`echo 100 ${percentfree} | awk ' { printf "%.2f\n", $1 - $2 } '`
	if [ ! -s ${TMPTBL}.read -a ! -s ${TMPFRAG}.read ]
	then
		echo "${dbspace}  ${amtfree}   ${percentused}%  (No tables in Dbspace)" >> ${DBSFILE}
		# No tables in dbspace (all indexes probably).
		# Prepare email lines if dbspace is less than 3% free.
		if [ ${percentfree} -lt 3 ]
		then
			echo "Space low in DBSpace ${dbspace} with no tables on `date`" >> ${MAILFILE}
			echo "for host ${HOST}, instance ${INFORMIXSERVER}." >> ${MAILFILE}
			echo "DBSpace ${dbspace} has ${percentfree} percent free space." >> ${MAILFILE}
			echo " " >> ${MAILFILE}
		fi
	else
		# Add lines to table space status portion of output "report".
		echo "${dbspace}  ${amtfree}   ${percentused}%  (Tables in Dbspace)" >> ${DBSFILE}
		echo " " >> ${TBLFILE}
		echo "Tables in DBSpace ${dbspace} with ${amtfree} pages remaining" >> ${TBLFILE}
		echo "Table Name  Alloc Pages   Used Pages   % Used   Next Extent Pages   Total Extents" >> ${TBLFILE}
		echo " " >> ${TBLFILE}
		if [ -s ${TMPTBL}.read ]
		then
			# Check non-fragmented tables in DBSpace, see if any next extent size will exhaust DBSpace remaining space.
			while read dbs tabname
			do
				dbaccess sysmaster << EOF 2>>checkspc.err | grep -v "^$" | grep -v "ti_nextns" | read NOEXT NEXTE ALLOC USED
select ti_nextns, ti_nextsiz, ti_nptotal, ti_npused
from systabinfo a, systabnames b
where a.ti_partnum = b.partnum
and b.dbsname = "$DBNAME"
and b.tabname = "$tabname";
EOF
				USEDPER=`echo ${USED} ${ALLOC} | awk ' { printf "%.2f\n", $1 * 100.00 / $2 } '`
				FREESP=`expr ${ALLOC} - ${USED}`
				FREEPER=`echo ${FREESP} ${ALLOC} | awk ' { printf "%.2f\n", $1 * 100.00 / $2 } '`
				echo "${tabname}  ${ALLOC}   ${USED}   ${USEDPER}%   ${NEXTE}   ${NOEXT}" >> ${TBLFILE}
				# If extent is more than 80% full and next extent is greater than available dbspace space,
				# then prepare email.  If more than 95% full, add warning message.
				if [ ${NEXTE} -ge ${amtfree} -a ${USEDPER} -gt 80 ]
        			then
					echo "Space is low in DBSpace ${dbs} with tables on `date`" >> ${MAILFILE}
					echo "for host ${HOST}, instance ${INFORMIXSERVER}." >> ${MAILFILE}
					echo "Table ${tabname} next extent of ${NEXTE} pages will use all free ${amtfree} pages in ${dbs}." >> ${MAILFILE}
					echo "Table has ${ALLOC} pages allocated, ${FREESP} pages free, and ${USEDPER} percent used." >> ${MAILFILE}
					if [ ${USEDPER} -ge 95 ]
					then
						echo "WARNING!!  DBSPACE CLOSE TO BEING USED UP BY A NEXT EXTENT!" >> ${MAILFILE}
						echo " " >> ${MAILFILE}
					else
						echo " " >> ${MAILFILE}
					fi
				fi
			done < ${TMPTBL}.read
		fi
		if [ -s ${TMPFRAG}.read ]
		then
			# Check fragmented tables in DBSpace to see if any next extent size will exhaust DBSpace remaining space.
			# The oncheck output contains all the fragments in its output, so need to first determine where to
			# start based on the dbspace currently being checked.
			while read dbs tabname partnum
			do
                                dbaccess sysmaster << EOF 2>>checkspc.err | grep -v "^$" | grep -v "ti_nextns" | read NOEXT NEXTE ALLOC USED
select ti_nextns, ti_nextsiz, ti_nptotal, ti_npused
from systabinfo a, systabnames b
where a.ti_partnum = b.partnum
and b.dbsname = "$DBNAME"
and b.tabname = "$tabname"
and b.partnum = $partnum;
EOF
				USEDPER=`echo ${USED} ${ALLOC} | awk ' { printf "%.2f\n", $1 * 100.00 / $2 } '`
				FREESP=`expr ${ALLOC} - ${USED}`
				FREEPER=`echo ${FREESP} ${ALLOC} | awk ' { printf "%.2f\n", $1 * 100.00 / $2 } '`
				echo "${tabname}  ${ALLOC}   ${USED}   ${USEDPER}%   ${NEXTE}   ${NOEXT}" >> ${TBLFILE}
				# If extent is more than 80% full and next extent is greater than available dbspace space,
				# then prepare email.  If more than 95% full, add warning message.
				if [ ${NEXTE} -ge ${amtfree} -a ${USEDPER} -gt 80 ]
        			then
					echo "Space is low in DBSpace ${dbs} with tables on `date`" >> ${MAILFILE}
					echo "for host ${HOST}, instance ${INFORMIXSERVER}." >> ${MAILFILE}
					echo "Table ${tabname} next extent of ${NEXTE} pages will use all free ${amtfree} pages in ${dbs}." >> ${MAILFILE}
					echo "Table has ${ALLOC} pages allocated, ${FREESP} pages free, and ${USEDPER} percent used." >> ${MAILFILE}
					if [ ${USEDPER} -ge 95 ]
					then
						echo "WARNING!!  DBSPACE CLOSE TO BEING USED UP BY A NEXT EXTENT!" >> ${MAILFILE}
						echo " " >> ${MAILFILE}
					else
						echo " " >> ${MAILFILE}
					fi
				fi
			done < ${TMPFRAG}.read
		fi
	fi
done < ${TMPSPC}

# Read blob page information and see if any are almost used up.  If so, prepare email.
if [ -s ${TMPSPC}.blob ]
then
	while read dbspace amtfree percentfree
	do
		percentused=`echo 100 ${percentfree} | awk ' { printf "%.2f\n", $1 - $2 } '`
		echo "${dbspace}  ${amtfree}   ${percentused}%  (Blob Dbspace)" >> ${DBSFILE}
		if [ ${percentfree} -lt 3 -a ${amtfree} -lt 20000 ]
		then
			echo "Space is low in blob DBSpace ${dbspace} on `date`" >> ${MAILFILE}
			echo "for host ${HOST}, instance ${INFORMIXSERVER}." >> ${MAILFILE}
			echo "The blob space ${dbspace} has only ${amtfree} remaining blobs," >> ${MAILFILE}
			echo "and it has only ${percentfree} space free." >> ${MAILFILE}
			echo "WARNING!!  BLOB SPACE CLOSE TO BEING USED UP!" >> ${MAILFILE}
			echo " " >> ${MAILFILE}
		fi
	done < ${TMPSPC}.blob
fi

# Send email if there is anything to report.
if [ -s ${MAILFILE} ]
then
	echo "Details are located in the ${LOG_DIR}/checkspc.out file." >> ${MAILFILE}
	mailx -m -s "Warning - possible space concerns for host ${HOST} on `date`" ${DBA_MAIL} < ${MAILFILE}
fi

# Add two prepared portions of "report" to output file.
cat ${DBSFILE} >> checkspc.out
echo " " 
cat ${TBLFILE} >> checkspc.out

rm -f ${DBSFILE}*
rm -f ${TBLFILE}*
rm -f ${TMPFILE}*
rm -f ${TMPSPC}*
rm -f ${TMPTBL}*
rm -f ${TMPFRAG}*
rm -f ${MAILFILE}*

# Write final lines to output "report" file.
echo " "
echo "End of lookatspace run for `date` for ${DBNAME}"
echo " "

} #do_lookatspace

################################################################################

# Determine if this is a multi-database or single database instance
NUMPARAMS=$#

if [ "${1}" = "MD" ]
then
        export MULTIDB="yes"
        shift 1
        NUMPARAMS=`expr ${NUMPARAMS} - 1`
else
        export MULTIDB="no"
fi

# Set up environment.

# Capture Informix information from .profile of informix ID and parameter.
INFHOME=`grep informix /etc/passwd | head -1 | cut -d: -f6`
grep INFORMIXDIR ${INFHOME}/.profile > ${INFHOME}/.tmpcks.$$
if [ ${NUMPARAMS} -eq 0 ]
then
        grep INFORMIXSERVER ${INFHOME}/.profile >> ${INFHOME}/.tmpcks.$$
        grep ONCONFIG ${INFHOME}/.profile >> ${INFHOME}/.tmpcks.$$
else
        export INFORMIXSERVER=${1}_shm
        export ONCONFIG=onconfig.${1}
fi
. ${INFHOME}/.tmpcks.$$
rm ${INFHOME}/.tmpcks.$$
export PATH=$PATH:$INFORMIXDIR/bin

# Set variables for use by this shell, including some temporary files.
DBA_MAIL="dba@s1.com"
HOST=`hostname`
if [ ${NUMPARAMS} -eq 0 ]
then
        LOG_DIR="${INFORMIXDIR}/logs"
else
        LOG_DIR="/db_activity/${1}/logs"
fi
TMP_DIR="/tmp"
DBSFILE=${TMP_DIR:-/tmp}/checkout_d.$$
TBLFILE=${TMP_DIR:-/tmp}/checkout_t.$$
TMPFILE=${TMP_DIR:-/tmp}/checktmp.$$
TMPSPC=${TMP_DIR:-/tmp}/checkspc.$$
TMPTBL=${TMP_DIR:-/tmp}/checktbl.$$
TMPFRAG=${TMP_DIR:-/tmp}/checkfrag.$$
MAILFILE=${TMP_DIR:-/tmp}/checkmail.$$

cd ${LOG_DIR}

# Send all output to existing file in LOG_DIR.
exec >> ${LOG_DIR}/checkspc.out 2>&1

if [ "${MULTIDB}" = "no" ]
then
#       Determine name of database to be checked by accessing sysmaster database
#       and getting the database name from the largest table in the instance.
	dbaccess sysmaster << EOF >>checkspc.out  2>checkspc.err
unload to /tmp/dbname.$$ delimiter ':'
select dbsname, ti_nrows
from systabinfo, systabnames
where partnum = ti_partnum
order by ti_nrows desc;
EOF
        export DBNAME=`head -1 /tmp/dbname.$$ | cut -d: -f1`
        rm /tmp/dbname.$$
        if [ "${DBNAME}" = "" ]
        then
                echo "No DBNAME for database from systabnames"
		echo "Lookatspace run aborted on `date`"
                exit 1
        fi
        do_lookatspace
else
	cat /dev/null > checkspc.err
        for dbn in `cat updstats.list`
        do
                DBNAME=${dbn}
                do_lookatspace
        done
fi

exit 0

