#!/bin/ksh
# updstats.sh 1.12 of 12/04/01
# @(#)updstats.sh	1.12
##############################################################################
# updstats.sh
##############################################################################
# Execute update statistics for the production database. 
#
# This shell is customized for each database server based mainly on how long 
# it takes to run the command.  Both the scheduled run times and what/how
# statistics are updated should be modified for each database server depending
# on the size of the database and application needs.
#
# Which tables and/or columns have statistics updated and in what manner
# (low, medium, or high) is determined by the first 3 SQL statements inside
# this shell.  The final SQL statement should always be run since it creates
# the procedure update stats.  The results of the 4 SQL statements are 
# combined into one SQL file which is run using dbaccess.  The use of these
# results can be overridden by using the "override" flag.
#
# Two or three parameters are passed to the shell.  The first is the instance
# name if this is a multi-instance server.  The second (or first) contains
# the value of the override flag ("yes" or "no"), and the third (or second)
# selects "full" or "low" statistics generation.
#
# Typically run from maestro two or three times per week for production.
##############################################################################

# Set up environment

# Capture Informix information from .profile of informix ID and parameter.
INFHOME=`grep informix /etc/passwd | head -1 | cut -d: -f6`
grep INFORMIXDIR ${INFHOME}/.profile > ${INFHOME}/.tmpupd.$$
if [ $# -eq 2 ]
then
        grep INFORMIXSERVER ${INFHOME}/.profile >> ${INFHOME}/.tmpupd.$$
        grep ONCONFIG ${INFHOME}/.profile >> ${INFHOME}/.tmpupd.$$
	export MULTI="no"
else
        export INFORMIXSERVER=${1}_shm
        export ONCONFIG=onconfig.${1}
	export DIRNAME=${1}
	export MULTI="yes"
	shift 1
fi
. ${INFHOME}/.tmpupd.$$
rm ${INFHOME}/.tmpupd.$$
export PATH=$PATH:$INFORMIXDIR/bin

# Update statistics SQL override flag passed to shell.  The default is
# "no". The update statistics SQL override flag is "yes" if updstats.sql
# is to be used in updating the statistics instead of the generated SQL.
if [ "${1}" = "no" ]
then
	export OVERRIDE="no"
else
	if [ "${1}" = "yes" ]
	then
		export OVERRIDE="yes"
	else
		export OVERRIDE="no"
	fi
fi

# Update statistics "full" or "low" flag passed to shell.  The default 
# is "full". The update statistics flag is "low" if only the "low" part
# of the generated SQL to update statistics is to be done.  It is "full"
# if all of the generated update statistics SQL is to be executed.
if [ "${2}" = "full" ]
then
	export TYPE="full"
else
	if [ "${2}" = "low" ]
	then
		export TYPE="low"
	else
		export TYPE="full"
	fi
fi

# Set PSORT_NPROCS environment variable to number of CPU's or 1 less 
# than number of CPU's if on a multi-instance server.
SARCPU=`sar -u -M 1 1 | wc -l`
if [ "${MULTI}" = "yes" ]
then
	export PSORT_NPROCS=`expr ${SARCPU} - 6`
else
	export PSORT_NPROCS=`expr ${SARCPU} - 5`
fi

DBA_MAIL="dba@s1.com"
HOST=`hostname`
if [ "${MULTI}" = "yes" ]
then
        LOG_DIR="/db_activity/${DIRNAME}/logs"
else
        LOG_DIR="${INFORMIXDIR}/logs"
fi

cd ${LOG_DIR}
rm -f updstat.out
rm -f updstat.err

exec >> ${LOG_DIR}/updstat.log 2>&1

echo "Update statistics started on `date`"

# Determine name of database to be checked by accessing sysmaster database
# and getting the database name from the largest table in the instance.
dbaccess sysmaster << EOF >>updstat.out  2>>updstat.err
unload to /tmp/dbname.$$ delimiter ':'
select dbsname, ti_nrows
from systabinfo, systabnames
where partnum = ti_partnum
order by ti_nrows desc;
EOF

export DBNAME=`head -1 /tmp/dbname.$$ | cut -d: -f1`
rm /tmp/dbname.$$
if [ "${DBNAME}" = "" ]
then
	echo "No DBNAME for database from systabnames"
	echo "Update statistics aborted on `date`"
	exit 1
fi
echo "DBNAME is ${DBNAME}" >> updstat.out

# Create update statistics SQL statements automatically.
# First, obtain which statistics to do using 3 SQL statements.
# What is selected may change for each installation by changing
# the where clause parameters in each SQL. 
dbaccess ${DBNAME} << EOF >>updstat.out  2>>updstat.err
output to "updstatmed.sql" without headings
select unique "update statistics medium for table " ||
  trim (tabname),
  " distributions only;"
from systables
where systables.tabid > 99 and
  systables.nrows < 50000;
output to "updstathigh.sql" without headings
select unique "update statistics high for table " ||
  trim (tabname),
  " (" || trim (colname) || ")",
  " distributions only;"
from systables, sysindexes, syscolumns
where systables.tabid = sysindexes.tabid and
  systables.tabid = syscolumns.tabid and
  sysindexes.part1 = syscolumns.colno and
  systables.tabid > 99;
output to "updstatlow.sql" without headings
select unique "update statistics low for table " ||
  trim (tabname) || ";"
from systables
where systables.tabid > 99;
EOF

# Combine the three files into one SQL file to run from dbaccess
# and add update statistics procedure to the SQL file.  The echo
# statement adds the update statistics for procedures.  For very
# large databases, these commands should be run in parallel.  Will
# need to change dbaccess statements below or create additional
# script(s) to handle parallel running.
echo "set pdqpriority low;" > updstatall.sql
cat updstatmed.sql updstathigh.sql updstatlow.sql >> updstatall.sql
echo "update statistics for procedure;" >> updstatall.sql

# Add update statistics for procedures to "low" SQL file also,
# in the event that this SQL is one chosen to be run.
echo "set pdqpriority low;" > updstattemp.sql
cat updstatlow.sql >> updstattemp.sql
echo "update statistics for procedure;" >> updstattemp.sql
mv updstattemp.sql updstatlow.sql

# Run dbaccess using this generated SQL to update statistics.
# Add additional statements using nohup if need to run in parallel.
# If override flag is set, use "hard coded" SQL file.
if [ "${OVERRIDE}" = "yes" ]
then
	dbaccess ${DBNAME} updstats >>updstat.out  2>>updstat.err
else
	if [ "${TYPE}" = "low" ]
	then
		dbaccess ${DBNAME} updstatlow >>updstat.out  2>>updstat.err
	else
		dbaccess ${DBNAME} updstatall >>updstat.out  2>>updstat.err
	fi
fi

rm -f updstatmed.sql updstathigh.sql
echo "Update statistics completed on `date`"

exit 0

